/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.consumer.throwable;

import aQute.bnd.annotation.ConsumerType;

import java.util.Objects;

/**
 * Defines a {@code Consumer} that can throw an exception.
 *
 * <p>
 * This interface can be implemented with a lambda function.
 * </p>
 *
 * @author Alejandro Hernández
 * @param  <A> the type of the first argument of the consumer
 */
@ConsumerType
@FunctionalInterface
public interface ThrowableConsumer<A> {

	/**
	 * Returns an empty {@code ThrowableConsumer} that doesn't perform any
	 * operation.
	 *
	 * @return an empty {@code ThrowableConsumer} that doesn't perform any
	 *         operation
	 */
	public static <A> ThrowableConsumer<A> empty() {
		return __ -> {
		};
	}

	/**
	 * Operates with one parameter and returns {@code void}. This function can
	 * be implemented explicitly or with a lambda.
	 *
	 * @param a the first function argument
	 */
	public void accept(A a) throws Exception;

	/**
	 * Returns the {@code ThrowableConsumer} function that first executes the
	 * current {@code ThrowableConsumer} instance's {@code accept} method, then
	 * executes the {@code after} parameter's {@code accept} method.
	 *
	 * @param  after the {@code ThrowableConsumer} instance to execute after the
	 *         current instance
	 * @return the {@code ThrowableConsumer} that executes the current
	 *         instance's {@code accept} method, as well as that of {@code
	 *         after}
	 */
	public default ThrowableConsumer<A> andThen(
		ThrowableConsumer<? super A> after) {

		Objects.requireNonNull(after);

		return a -> {
			accept(a);
			after.accept(a);
		};
	}

}