/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.consumer.throwable;

import aQute.bnd.annotation.ConsumerType;

import java.util.Objects;

/**
 * Defines a {@code TriConsumer} that can throw an exception.
 *
 * <p>
 * This interface can be implemented with a lambda function.
 * </p>
 *
 * @author Alejandro Hernández
 * @param  <A> the type of the first argument of the consumer
 * @param  <B> the type of the second argument of the consumer
 * @param  <C> the type of the third argument of the consumer
 */
@ConsumerType
@FunctionalInterface
public interface ThrowableTriConsumer<A, B, C> {

	/**
	 * Returns an empty {@code ThrowableTriConsumer} that doesn't perform any
	 * operation.
	 *
	 * @return an empty {@code ThrowableTriConsumer} that doesn't perform any
	 *         operation
	 */
	public static <A, B, C> ThrowableTriConsumer<A, B, C> empty() {
		return (a, b, c) -> {
		};
	}

	/**
	 * Operates with three parameters and returns {@code void}. This function
	 * can be implemented explicitly or with a lambda.
	 *
	 * @param a the first function argument
	 * @param b the second function argument
	 * @param c the third function argument
	 */
	public void accept(A a, B b, C c) throws Exception;

	/**
	 * Returns the {@code ThrowableTriConsumer} function that first executes the
	 * current {@code ThrowableTriConsumer} instance's {@code accept} method,
	 * then executes the {@code after} parameter's {@code accept} method.
	 *
	 * @param  after the {@code ThrowableTriConsumer} instance to execute after
	 *         the current instance
	 * @return the {@code ThrowableTriConsumer} that executes the current
	 *         instance's {@code accept} method, as well as that of {@code
	 *         after}
	 */
	public default ThrowableTriConsumer<A, B, C> andThen(
		ThrowableTriConsumer<? super A, ? super B, ? super C> after) {

		Objects.requireNonNull(after);

		return (a, b, c) -> {
			accept(a, b, c);
			after.accept(a, b, c);
		};
	}

}