/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.function.throwable;

import aQute.bnd.annotation.ConsumerType;

import java.util.Objects;

/**
 * Defines a {@code java.util.function.Function} that can throw an exception.
 *
 * @author Alejandro Hernández
 * @param  <A> the type of the argument of the function
 * @param  <R> the type of the result of the function
 */
@ConsumerType
@FunctionalInterface
public interface ThrowableFunction<A, R> {

	/**
	 * Returns a composed function that first applies the current {@code
	 * ThrowableFunction} instance to its input, and then applies the {@code
	 * throwableFunction} to the result.
	 *
	 * @param  throwableFunction the function to apply after applying the
	 *         current {@code ThrowableFunction} instance
	 * @return the composed function
	 */
	public default <V> ThrowableFunction<A, V> andThen(
		ThrowableFunction<? super R, ? extends V> throwableFunction) {

		Objects.requireNonNull(throwableFunction);

		return (A a) -> throwableFunction.apply(apply(a));
	}

	/**
	 * Returns the result of applying the current {@code ThrowableFunction}
	 * instance to the argument.
	 *
	 * @param  a the argument
	 * @return the result of applying the current {@code ThrowableFunction}
	 *         instance to the argument
	 */
	public R apply(A a) throws Exception;

}